% Mass of the arm
m = 1;
% Length of the arm
arm_length = 0.3;
% Inertia
I = m *(arm_length/2)^2;

% Gravity
g = 9.81;

% Damping coefficient for the forward model
dampCoeff =0.5;
% Sensory delay (s)
delay =  0.125;
%delay =  0.1;
% Simulation time step (s)
dt = 0.001;
% Simulation duration (s)
max_time = 2;

% Time to start the movement (s)
mvt_start_time = 0.5;

% Delayed versions of joint angle and velocity
delay_index = delay/dt+1;
delayed_ang = zeros(1,delay_index);
delayed_vel = zeros(1,delay_index);

% Plant - current actual joint angle and velocity
ang_actual = 0;
vel_actual = 0;

% Forward model - estimated current joint angle and velocity
ang_estimated = 0;
vel_estimated = 0;

% Target joint angle
final_target_ang =pi/2;

% History of joint angle, delayed joint angle, and torque
ang_history = [];
ang_del_history = [];
ang_target_history = [];
torque_history = [];
pert_history = [];

% Perturbation start time (s)
pert_start_time = .75;
% Perturbation end time (s)
pert_end_time = 1;
% Perturbation amplitude
pert_amp =1;
%pert_amp =0;

% Gain and damping parameters for forward model
Kp_forward = 5;
%Kp_forward = 0;
Kd_forward = 0.3;
%Kd_forward = 0;

% Gain and damping parameters for feedback model
Kp_feedback = 5;
%Kp_feedback = 0;
Kd_feedback = 0.3;
%Kd_feedback = 0;

for t =0:dt:max_time

    % Set the desired joint angle once the movement start time is reached
    if t <  mvt_start_time
        target_ang = 0;
    else
        target_ang = final_target_ang;
    end

    % forward model torque
    forward_torque = Kp_forward * (target_ang - ang_estimated) + Kd_forward * ( - vel_estimated);
    % feedback torque delayed
    feedback_torque = Kp_feedback * (target_ang - delayed_ang(delay_index)) + Kd_feedback * ( - delayed_vel(delay_index));
    
    % Total torque
    total_torque=forward_torque + feedback_torque;

    %forward model of arm_dynamics
    acc_estimated = (total_torque - dampCoeff*vel_estimated ) / I ;
    vel_estimated = vel_estimated + dt * acc_estimated;
    ang_estimated = ang_estimated + dt * vel_estimated;

    % Apply perturbation
    if ((t >  pert_start_time) && (t < pert_end_time))
        pert = pert_amp;
    else
        pert = 0;
    end

    % Plant - forward model of the arm
    acc = (total_torque - dampCoeff*vel_actual + pert) / I ;
    vel_actual = vel_actual + dt * acc;
    ang_actual = ang_actual + dt * vel_actual;

    % Update delayed copy of joint angle and velocity
    for i = delay_index:-1:1
        delayed_ang(i+1) = delayed_ang(i);
        delayed_vel(i+1) = delayed_vel(i);
    end
    delayed_ang(1) = ang_actual;
    delayed_vel(1) = vel_actual;
    
    % Update histories
    ang_history(end+1) = ang_actual;
    ang_del_history(end+1) = delayed_ang(delay_index);
    torque_history(end+1) = total_torque;
    ang_target_history(end+1)=target_ang;
    pert_history(end+1) = pert;
end

figure()
subplot(2,1,1)
plot(0:dt:max_time, ang_history)
hold on
plot(0:dt:max_time, ang_del_history, 'k')
plot(0:dt:max_time, ang_target_history, '--');
plot(0:dt:max_time, pert_history, 'r');
legend('angle','delayed angle','target angle','perturbation');
xlabel('time')
ylabel('angle')
subplot(2,1,2)
plot(0:dt:max_time, torque_history)
xlabel('time')
ylabel('motor command')
