% learning a simple forward model of an arm
% the arm is simply a bar moving in 1D with intertia and damping
% Nicolas Schweighofer

clear all
clf

% time parameters
dt = 0.005;
max_time = 2;
mvt_start_time = 0.5;

% arm model
m = 1;
arm_length = 0.3;
I = m *(arm_length/2)^2;
dampCoeff =0.5;

% controller parameters
Kp = 30;
Kv = 2;
desired_end_pos =pi/2;

% learning model parameters
w1 = 0;
w2 = 0; 
% learning rate
alpha = 0.0001;

% weight histories
w1_hist=[];
w2_hist=[];

% learning loop
for i = 1:20

    vel = 0;
    pos = 0;
    vel_estimated = 0;
    pos_estimated = 0;

    index_time =0;

    % movement loop

    for t =0:dt:max_time

        if t <  mvt_start_time
            desired_pos = 0;
        else
            desired_pos = desired_end_pos;
        end

        % feedback torque
        feedback_torque =  Kp * (desired_pos - pos) + Kv * (- vel);

        % arm forward dynamics 
        acc = (feedback_torque - dampCoeff*vel) / I ;
        vel = vel + dt * acc;
        pos = pos + dt * vel;

        %forward model of arm_dynamics
        acc_estimated = w1*feedback_torque + w2*vel_estimated;
        vel_estimated = vel_estimated + dt * acc_estimated;
        pos_estimated = pos_estimated + dt * vel_estimated;

        % learning the weights with gradient descent
        error = (acc  - acc_estimated);
        w1 = w1 + alpha*error*feedback_torque;
        w2 = w2 + alpha*error*vel_estimated;
    
        % plotting    
        index_time = index_time + 1;
        acc_estim_plot(index_time) = acc_estimated;
        acc_plot(index_time) = acc;
        vel_estim_plot(index_time) = vel_estimated;
        vel_plot(index_time) = vel;    
        torque_plot(index_time) = feedback_torque;        
    end

    if i==1
        figure(1)
        subplot(2,1,1)
        plot(0:dt:max_time, acc_plot)
        xlabel('time')
        ylabel('acc')
        hold on
        plot(0:dt:max_time, acc_estim_plot, 'k')
        legend('acc','estimated acc');
        title('First trial');

        subplot(2,1,2)
        plot(0:dt:max_time, vel_plot)
        xlabel('time')
        hold on
        plot(0:dt:max_time, vel_estim_plot, 'k')
        ylabel('vel')
        legend('vel','estimated vel');
    end

    w1_hist=[w1_hist w1];
    w2_hist=[w2_hist w2];
end

figure(2)
subplot(2,1,1)
plot(0:dt:max_time, acc_plot)
xlabel('time')
ylabel('acc')
hold on
plot(0:dt:max_time, acc_estim_plot, 'k')
legend('acc','estimated acc');
title('Last trial');

subplot(2,1,2)
plot(0:dt:max_time, vel_plot)
xlabel('time')
hold on
plot(0:dt:max_time, vel_estim_plot, 'k')
ylabel('vel')
legend('vel','estimated vel');

figure(3);
plot(w1_hist,'b');
hold on;
plot(w2_hist,'r');
legend('w1','w2');
xlabel('Trial');
ylabel('Weight');
